package IOulm;

import java.io.*;
import java.util.Vector;

/**
 * Class for simplifying read operations and make them similar to those
 * used in the Ulmer Oberon system
 *
 * @see      Write
 * @author   nh @ SAI
 * @version  0.5
 */

// Urc = Ulmer Reader Class
/* Mit der Bezeichnung "Urc" soll zum Ausdruck gebracht werden, dass es sich
   hierbei _nicht_ um eine Standard-Java-Klasse handelt, sondern um
   eine "Eigenerfindung" aus Ulm.
   Der Name "Read" (in Analogie zu dem Modul "Read" in Oberon) soll hierfuer
   deshalb nicht verwendet werden, da das Einlesen nicht ganz gleich zu
   dem in Oberon funktioniert.

   Changelog:
   ----------
   0.2: Um mehrere Zahlen in einer Eingabezeile stehen haben zu knnen,
        sammeln wir erst den gesamten Eingabestring ein, zerlegen ihn
	am Whitespace-Zeichen und speichern diese Teilstrings in einem
	Vektor ab.
	Dieser Vektor dient als FIFO-Puffer, d.h., es wird erst wieder
	von der Standardeingabe gelesen, wenn alle Zahlen aus dem Puffer
	abgefordert wurden.
	(Dies gilt fr Integer und Real-Zahlen!)

   0.3: Fhrende Blanks vor Zahlen werden nun ignoriert (fhrten bislang
        dazu, da die Zahl nicht als solche erkannt wurde)
	(Dies gilt fr Integer und Real-Zahlen!)

   0.4: Ab jetzt werden auch Chars in einem Vektor "gepuffert".
        Auerdem stehen Methoden zur Verfgung, um die jeweiligen
	Integer-, Real- und Char-Vektoren zu leeren (clearXXXBuf,
	mit XXX = Int, Real oder Char).

   0.5: Pufferung von Chars (aus 0.4) wieder entfernt, da das Einlesen
        nun nicht mehr via stdin.readLine() erfolgt sondern via
	stdin.read(); somit wird nun _genau_ ein Zeichen gelesen, soda
	der "Umweg" ber die Pufferung in einem Vektor nicht mehr erforderlich
	ist
*/

public class Urc {

   private static LineNumberReader stdin = new LineNumberReader
                                           (new InputStreamReader(System.in));
   private static Vector intVec = new Vector();
   private static int numberInt = 0;
   private static Vector dblVec = new Vector();
   private static double numberDouble = 0;
   private static char ch = 0;
   private static String string = "";

   /**
    * read one or more integer value(s) from stdin
    * <p>
    * This method tries to read one or more integer value(s) from the
    * standard input.
    * <p>
    * All values read are buffered, so no input is read from
    * the standard input until the buffer is empty.
    * <p>
    * The value(s) read can be retrieved by the getInt()-Method,
    * (respecting the FIFO-principle (First in, first out)).
    *
    * @see     getInt()
    * @see     clearIntBuf()
    * @return  true if reading was successfull, otherwise false 
    */
   public static boolean readInt() {
      if (intVec.size() == 0) {
         try {
            String[] result = stdin.readLine().trim().split("\\s");
            for (int x=0; x<result.length; x++)
                       intVec.add(result[x].trim());
         }
         catch(Exception e) {
            return false;
         }
      }
      try {
	 numberInt = Integer.parseInt((String)intVec.remove(0));
      }
      catch(Exception e) {
	 return false;
      }
      return true;
   }

   /**
    * returns the integer value last read
    * <p>
    * This method returns the integer value last read or '0' in case
    * that no value was read yet.
    *
    * @see    readInt()
    * @see    clearIntBuf()
    * @return the integer value last read or '0' in case that no value
              was successfully read yet.
    */
   public static int getInt() {
      return numberInt;
   }

   /**
    * removes all elements from the integer buffer
    *
    * @see    readInt()
    * @see    getInt()
   */
   public static void clearIntBuf() {
      intVec.removeAllElements();
   }

   /**
    * read one or more real value(s) from stdin
    * <p>
    * This method tries to read one or more real value(s) from the
    * standard input.
    * The value(s) read can be retrieved by the getReal()-Method,
    * (respecting the FIFO-principle (First in, first out)).
    *
    * @see     getReal()
    * @see     clearRealBuf()
    * @return  true if reading was successfull, otherwise false 
    */
   public static boolean readReal() {
      if (dblVec.size() == 0) {
         try {
            String[] result = stdin.readLine().trim().split("\\s");
            for (int x=0; x<result.length; x++) 
                       dblVec.add(result[x].trim()); 
         }
         catch(Exception e) {
            return false;
         }
      }
      try {
	 numberDouble = Double.parseDouble((String)dblVec.remove(0));
      }
      catch(Exception e) {
	 return false;
      }
      return true;
   }

  /**
   * returns the real value last read
   * <p>
   * This method returns the real value last read or '0' in case
   * that no value was read yet.
   *
   * @see    readReal()
   * @see    clearRealBuf()
   * @return the real value last read or '0' in case that no value
             was successfully read yet.
   */
   public static double getReal() {
      return numberDouble;
   }

   /**
    * removes all elements from the real value buffer
    *
    * @see    readReal()
    * @see    getReal()
   */
   public static void clearRealBuf() {
      dblVec.removeAllElements();
   }

   /**
    * read (exactly) one character from stdin
    * <p>
    * This method tries to read (exactly) one character from the 
    * standard input.
    *
    * @see     getChar()
    * @return  true if reading was successfull, otherwise false 
    */
   public static boolean readChar() {

    try {
        int i = stdin.read();
        if ( i < 0 ) return false;
        ch = (char) i;
        return true;
     }
     catch(Exception e) {
            return false;
     }
   }

  /**
    * returns the character last read
    * <p>
    * This method returns the character last read or '0' in case
    * that none was read yet.
    *
    * @see    readChar()
    * @return the character last read or '0' in case that no value
              was successfully read yet.
    */
   public static char getChar() {
      return ch;
   }

   /**
    * read a string value from stdin
    * <p>
    * This method tries to read a string from the standard input.
    * The string read can be retrieved by the getString()-Method.
    *
    * @see     getString()
    * @return  true if reading was successfull, otherwise false 
    */
   public static boolean readString() {
      try {
         string = stdin.readLine();
      }
      catch(Exception e) {
	 return false;
      }
      return string != null;
   }

   /**
    * returns the string last read
    * <p>
    * This method returns the string last read or 'null' in case
    * that no string was read yet.
    *
    * @see    readString()
    * @return the string last read or 'null' in case that no string
              was successfully read yet.
    */
   public static String getString() {
      return string;
   }
}
